/*
	 *	Emptool				<file: scan.c>
	 *	
	 *	Written by HJ. Visscher.
	 *
	 *	ScanDump ()
	 *		Scans the output of "dump" that empire is printing
	 *
	 *	ScanNation ()
	 *		Scans the output of "nation" that empire is printing
	 *
	 *	ScanMap ()
	 *		Scans the output of "map" that empire is printing
	 */

#include "type.h"
#include "main.h"
#include "sector.h"
#include "nation.h"

void ScanSatellite ();

char *DelControl (s)
     char *s;
{
	register char *ptr;
	register char *bptr;
	static char buffer [BUFSIZ];

	bptr = buffer;
	ptr = s;

	while (*ptr != '\0')
	{
		*ptr &= 0x7F;
		if (*ptr < ' ')
		{
			*bptr++ = '^';
			*bptr++ = *ptr++ - ' ' + 'A';
		}
		else
			*bptr++ = *ptr++;
	}

	*bptr = '\0';

	return buffer;
}

char *DelTabs (s)
     char *s;
{
	int i;
	register char *ptr;
	register char *bptr;
	static char buffer [BUFSIZ];

	if (s == (char *) 0 || *s == '\0')
		return s;

	i = 0;
	ptr = s;
	bptr = buffer;

	while (*ptr != '\0')
		if (*ptr == '\t')
		{
			do
			{
				* (bptr ++)  = ' ';
				i ++;
			} while ((i % 8) != 0);
			ptr ++;
		}
		else
		{
			* (bptr ++) = * (ptr ++);
			i ++;
		}

	*bptr = '\0';

	return buffer;
}

char * DelNewline (str)
     char *str;
{
	char *p;

	if ((p = strchr (str, '\n')) != (char *) 0)
		*p = '\0';
	
	return str;
}

void SkipPtr (pptr, toskip)
     char **pptr, *toskip;
{
	if (*toskip == '^')
		while ((strchr (toskip + 1, **pptr) == (char *) 0) &&
							 **pptr != '\0')
			(*pptr) ++;
	else
		while ((strchr (toskip, **pptr) != (char *) 0) &&
							**pptr != '\0')
			(*pptr) ++;
}

void SkipBlank (pptr)
     char **pptr;
{
	SkipPtr (pptr, " \t");
}

bool ScanWordQuot (pptr, word)
     char **pptr;
     char *word;
{
	char buffer [BUFSIZ];
	char *ptr;
	bool quot = False;

	ptr = (word == (char *) 0) ? buffer : word;
	SkipBlank (pptr);

	if (**pptr == '\0')
		return False;

	while (**pptr != '\0' && ((**pptr != ' ' && **pptr != '\t') || quot))
	{
		if (**pptr == '\\')
		{
			if (*++(*pptr) == '\0')
				return False;
			*ptr++ = **pptr;
		}
		else if (**pptr == '"')
			quot = ! quot;
		else
			*ptr++ = **pptr;
		(*pptr) ++;
	}

	*ptr = '\0';
	return ! quot;
}

bool ScanWord (pptr, word)
     char **pptr;
     char *word;
{
	char buffer [BUFSIZ];
	char *ptr;

	ptr = (word == (char *) 0) ? buffer : word;
	SkipBlank (pptr);

	if (**pptr == '\0')
		return False;

	while (**pptr != '\0' && **pptr != ' ' && **pptr != '\t')
	{
		*ptr = **pptr;
		ptr ++;
		(*pptr) ++;
	}

	*ptr = '\0';
	return True;
}

bool ScanFmt (pptr, word, fmt)
     char ** pptr;
     char * word;
     char * fmt;
{
	char buffer [BUFSIZ];
	char * ptr;

	ptr = (word == (char *) 0) ? buffer : word;
	SkipBlank (pptr);
	while (CharInFormat (** pptr, fmt))
	{
		* ptr = ** pptr;
		ptr ++;
		(* pptr) ++;
	}

	* ptr = '\0';
	return True;
}

bool ScanDigit (pptr, num)
     char **pptr;
     int *num;
{
	int negated = 0;
	register int value = 0;

	SkipBlank (pptr);

	if (**pptr == '-')
	{
		negated = 1;
		(*pptr) ++;
	}
	else if (**pptr == '+')
		(*pptr) ++;

	if (**pptr < '0' || **pptr > '9')
		return False;

	while (**pptr >= '0' && **pptr <= '9') 
	{

		value = 10 * value + **pptr - '0';
		(*pptr) ++;
	}

	*num =  negated ? -value : value;

	return True;
}

static double ScanFloat (ptr)
     char *ptr;
{
	double p;

	(void) sscanf (ptr, "%lf", &p);
	return p;
}

void FScanNation ()
{
	char *ptr, *ind;
	int num, i;
	extern int capitol_x, capitol_y;

	FeedEmpire ("nation", PRINT);
	MessageAtEmpire ("scanning nation");

	(void) SkipNLines (3, DONT_PRINT);
	ptr = ReadEmpire (DONT_PRINT);
	if (ptr == (char *) 0) return;
	if (sscanf (ptr, "%*d%% eff capital at %d,%d",
					&capitol_x, &capitol_y) != 2)
		if (strncmp (ptr, "No capital", 10) == 0)
			capitol_x = capitol_y = 0;
		else
			Error ("Bad Nation output");

		/* scan realms */
	i = 0;
	for (;;)
	{
		ptr = ReadEmpire (DONT_PRINT);

		if (ptr == (char *) 0) return;

		if (* ptr != '#')
			break;

		if (sscanf (ptr, "# %*d %s # %*d %s # %*d %s",
					nation-> realm [i],
					nation-> realm [i + 1],
					nation-> realm [i + 2]) != 3) {
			Error ("Illegal realm, try with less realms");
		}

		i += 3;
	}

	nr_realms = i;

	ind = index (ptr, '$');
	n_treas (nation) = ScanFloat (ind + 1);

	ind = index (ptr, ':') + 1;

	if (!ScanDigit (&ind, &num))
		Error ("Bad nation output");

	n_mil_res(nation) = (long) num;

	ptr = ReadEmpire (DONT_PRINT);
	if (ptr == (char *) 0) return;
	n_educ (nation) = ScanFloat (ptr + 19);
	if ((ptr = strchr (ptr, 'H')) != (char *) 0)
		n_hap (nation) = ScanFloat (ptr + 16);

	ptr = ReadEmpire (DONT_PRINT);
	if (ptr == (char *) 0) return;
	n_tech (nation) = ScanFloat (ptr + 19);
	if ((ptr = strchr (ptr, 'R')) != (char *) 0)
		n_research (nation) = ScanFloat (ptr + 16);

	ptr = ReadEmpire (DONT_PRINT);
	if (ptr == (char *) 0) return;
	n_tech_fact (nation) = ScanFloat (ptr + 19);
	n_plag_fact (nation) = ScanFloat (ptr + 48);

	(void) WaitForPrompt (PRINT);
}

	/*
	 *	ScanDump returns the # of sectors read
	 */

int FScanDump (part, flag)
char *part;
int flag;
{
	int nr_s;

	FeedEmpire (Fmt ("dump %s", part), flag);
	nr_s = ScanDump (flag);
	(void) WaitForPrompt (flag);

	return nr_s;
}

int ScanDump (flag)
int flag;
{
	register Sector sct;
	char *ptr, *line, des;
	int lx, ly;
	int owner;
	int sectors_read;
	bool err;

#ifndef lint
	int nr; 
	int eff, mob, min, gol, fer, oil, ura, wor, ter;
	char dist_path [20];
	int u_cut, f_cut, s_cut, g_cut, p_cut, i_cut,
	      d_cut, b_cut, o_cut, l_cut, h_cut, r_cut;
	char  u_del, f_del, s_del, g_del, p_del, i_del,
	      d_del, b_del, o_del, l_del, h_del, r_del;
	int deltox, deltoy;
#endif lint


	if (flag == PRINT)
		MessageAtEmpire ("scanning dump");

	sectors_read = 0;

	(void) SkipNLines (1, DONT_PRINT);

	if (EmpireStatus () != E_PRINTING)
		return 0;

	ptr = ReadEmpire (DONT_PRINT);
	if (ptr == (char *) 0) return 0;

	SkipBlank (&ptr);
	if (strncmp (ptr, "DUMP", 4))
		/* no sector to scan */
		return 0;


	ptr = ReadEmpire (DONT_PRINT);
	if (ptr == (char *) 0) return 0;

	deity = (*ptr == 'o');

	for (;;)
	{
		line = ptr = ReadEmpire (DONT_PRINT);

		if (ptr == (char *) 0 || *ptr == '\0')
			break;

		if (deity)
			if (!ScanDigit (&ptr, &owner))
			{
				PrintAtEmpire (
				"Warning: Expected country num in dump");
				PrintAtEmpire (Fmt ("Skipping \"%s\"", line));
				Bell ();
				(void) EmpireMore ();
				continue;
			}

		err = ! ScanDigit (&ptr, &lx);
		err = err || ! ScanDigit (&ptr, &ly);

		if (err)
		{
			if (strncmp (ptr, "sector", 6) == 0)
				/* End of Dump */
				break;
			PrintAtEmpire ("Warning: Expected coordinats in dump");
			PrintAtEmpire (Fmt ("Skipping \"%s\"", line));
			Bell ();
			(void) EmpireMore ();
			continue;
		}
			

		sct = World (lx, ly, S_OWNED);

		if (deity)
			set_owner (sct, owner);
		else
			set_owner (sct, my_cnum);

		SkipBlank (&ptr);
		des = *ptr;

/* 
 *		Why? HJV
 *
 * 		if (! UpdateSectorDes (sct, des, True))
 * 		{
 * 			PrintAtEmpire (Fmt ("Skipping \"%s\"", line));
 * 			(void) EmpireMore ();
 * 			continue;
 * 		}
 */

		sectors_read ++;
		set_des (sct, des);

		/*
		 * lint complains & quits: 
		 *
		 * pass 2 error:(file scan.c) more than 67 args?
		 *
		 */
#ifndef lint
	/*
	 *	You should NOT do it this way, but I'm afraid it
	 *	takes too long when you read it in dummy vars
	 */


/* nds eff mob occ min gol fer
 * oil ura wor ava ter civ mil d(uw) d(foo) d(she) d(gun)
 * d(pet) d(iro) d(dus) d(bar) d(oil) d(lcm) d(hcm) d(rad) udel fdel sdel gdel
 * pdel idel ddel bdel odel ldel hdel rdel ucut fcut dcut gcut pcut
 * icut dcut bcut ocut lcut hcut rcut dpath dxc dyc tuw
 * tfoo tshe tgun tpet tiro tdus tbar toil tlcm thcm trad 
 */

		 nr = sscanf (ptr + 2, " %c %hd %d %c %d %d %d \
		      %d %d %d %hd %d %hd %hd %hd %hd %hd %hd \
		      %hd %hd %hd %hd %hd %hd %hd %hd %c %c %c %c \
		      %c %c %c %c %c %c %c %c %d %d %d %d %d \
		      %d %d %d %d %d %d %d %s %d %d %hd \
		      %hd %hd %hd %hd %hd %hd %hd %hd %hd %hd %hd",
				& sct-> sct2-> sct3-> nds,
				& sct-> sct2-> eff,
				& mob,
				& sct-> sct2-> sct3-> occ,
				& min,
				& gol,
				& fer,
				& oil,
				& ura,
				& wor,
				& sct-> sct2-> sct3-> ava,
				& ter,
				& sct-> sct2-> civ,
				& sct-> sct2-> mil,
				& sct-> sct2-> qua [D_UW],
				& sct-> sct2-> qua [D_FOO],
				& sct-> sct2-> qua [D_SHE],
				& sct-> sct2-> qua [D_GUN],
				& sct-> sct2-> qua [D_PET],
				& sct-> sct2-> qua [D_IRO],
				& sct-> sct2-> qua [D_DUS],
				& sct-> sct2-> qua [D_BAR],
				& sct-> sct2-> qua [D_OIL],
				& sct-> sct2-> qua [D_LCM],
				& sct-> sct2-> qua [D_HCM],
				& sct-> sct2-> qua [D_RAD],
				& u_del,
				& f_del,
				& s_del,
				& g_del,
				& p_del,
				& i_del,
				& d_del,
				& b_del,
				& o_del,
				& l_del,
				& h_del,
				& r_del,
				& u_cut,
				& f_cut,
				& s_cut,
				& g_cut,
				& p_cut,
				& i_cut,
				& d_cut,
				& b_cut,
				& o_cut,
				& l_cut,
				& h_cut,
				& r_cut,
				dist_path,
				& deltox,
				& deltoy,
				& sct-> sct2-> sct3-> tre [D_UW],
				& sct-> sct2-> sct3-> tre [D_FOO],
				& sct-> sct2-> sct3-> tre [D_SHE],
				& sct-> sct2-> sct3-> tre [D_GUN],
				& sct-> sct2-> sct3-> tre [D_PET],
				& sct-> sct2-> sct3-> tre [D_IRO],
				& sct-> sct2-> sct3-> tre [D_DUS],
				& sct-> sct2-> sct3-> tre [D_BAR],
				& sct-> sct2-> sct3-> tre [D_OIL],
				& sct-> sct2-> sct3-> tre [D_LCM],
				& sct-> sct2-> sct3-> tre [D_HCM],
				& sct-> sct2-> sct3-> tre [D_RAD]);

		if (nr != 65)
		{
			PrintAtEmpire ("Bad dump Format");
			PrintAtEmpire (Fmt ("Skipping \"%s\"", line));
			Bell ();
			(void) EmpireMore ();
		}

		set_mob (sct, mob);
		set_min (sct, min);
		set_gol (sct, gol);
		set_fer (sct, fer);
		set_oil (sct, oil);
		set_ura (sct, ura);
		set_ter (sct, ter);
		set_wor (sct, wor);

		set_dc_uw  (sct, u_del, u_cut);
		set_dc_foo (sct, f_del, f_cut);
		set_dc_she (sct, s_del, s_cut);
		set_dc_gun (sct, g_del, g_cut);
		set_dc_pet (sct, p_del, p_cut);
		set_dc_iro (sct, i_del, i_cut);
		set_dc_dus (sct, d_del, d_cut);
		set_dc_bar (sct, b_del, b_cut);
		set_dc_oil (sct, o_del, o_cut);
		set_dc_lcm (sct, l_del, l_cut);
		set_dc_hcm (sct, h_del, h_cut);
		set_dc_rad (sct, r_del, r_cut);

		set_dxc (sct, EX (deltox));
		set_dyc (sct, EX (deltoy));

		if (* dist_path != '_')
			SetDistPath (sct, dist_path);
			
		if (s_nds (sct) == '_')
			s_nds (sct) = s_des (sct);

		DEL_INFO (sct);
		SET_INFO_ALL (sct);
#endif lint
	}

	return sectors_read;
}

void FScanMap (part, flag)
     char *part;
     int flag;
{
	FeedEmpire (Fmt ("map %s", part), flag);
	ScanMap (flag);
	(void) WaitForPrompt (flag);
}

void FScanBmap (part, flag)
     char *part;
     int flag;
{
	FeedEmpire (Fmt ("bmap %s", part), flag);
	ScanMap (flag);
	(void) WaitForPrompt (flag);
}

void ScanMap (flag)
     int flag;
{
	register char *line;
	char *ptri;
	int first, second, len;
	char first1, first2, second1, second2;
	int xcoord, ycoord, length;
	bool first_line;
	register int i;
	register Sector sct;

	if (flag == PRINT)
		MessageAtEmpire ("scanning map");

	line = ptri = ReadEmpire (DONT_PRINT);
	if (line == (char *) 0) return;
	if (EmpireStatus () == E_COMMAND) {
		if (flag == PRINT)
		{
			PrintAtEmpire ("");
			PrintAtEmpire (line);
		}
		Message ("map command failed");
		Bell ();
		return;
	}

	len = strlen (line);

	SkipBlank (&ptri);

	second1 = second2 = '\0';
	first = second = 0;

	first1 = *ptri++;
	length = 1;

	if (*ptri != '\0' && *ptri != '\n')
	{
		second1 = *ptri++;
		length++;
	}

	while (*ptri != '\0' && *ptri != '\n')
	{
		ptri ++;
		length ++;
	}

	line = ptri = ReadEmpire (DONT_PRINT);
	if (line == (char *) 0) return;
	if (EmpireStatus () == E_COMMAND) {
		if (flag == PRINT)
		{
			PrintAtEmpire ("");
			PrintAtEmpire (line);
		}
		Message ("map command failed");
		Bell ();
		return;
	}

	SkipBlank (&ptri);
	first2 = *ptri++;

	if (*ptri != '\0' && *ptri != '\n')
		second2 = *ptri;

	if (first1 == '-')
		first = '0' - first2;
	else
	{
		first = 10 * (first1 - '0') + (first2 - '0');
		
		if (second1 != '\0')
		{
			if (second1 == '-')
				second = '0' - second2;
			else 
				second = 10 * (second1 - '0') + (second2 - '0');

			if (first > second)
				first = -first;
		}
	}

	first_line = True;
	for (;;)
	{
		ptri = line = ReadEmpire (DONT_PRINT);

		if (line == (char *) 0)
			return;

		if (EmpireStatus () != E_PRINTING)
		{
			if (flag == PRINT)
			{
				PrintAtEmpire ("");
				PrintAtEmpire (line);
			}
			return;
		}
			/* the hunderds */

		if (first_line)
		{
			char *ptr;

			first_line = False;
			ptr = ptri + 5;
			if (* ptr >= '0' && * ptr <= '9')
			{
				if (first < 0) {
					first = 10 * first + ('0' - * ptr);
					continue;
				}
				first = 10 * first + (* ptr - '0');
				if (second < 0 || second1 == '\0')
					continue;

				if (*++ptr != '\0' && *ptr != '\n') {
					second = 10 * second + (* ptr - '0');

					if (first > second)
						first = -first;
				}
				continue;
			}
		}

		if (line [3] == ' ' || strlen (line) < len)
			continue;

		if (! ScanDigit (&ptri, &ycoord))
		{
			PrintAtEmpire ("Map : Coordinate expected in map");
			PrintAtEmpire ( Fmt ("Skipping \"%s\"", line));
			Bell ();
			(void) EmpireMore ();
			continue;
		}

		for (i = ABS (ycoord + first) % 2; i < length; i += 2)
			if (line [5 + i]  != ' ')
			{
				xcoord = first + i;
				sct = World (xcoord, ycoord, S_DESIG);

				if (!UpdateSectorDes (sct, line [5 + i], True))
				{
					PrintAtEmpire (Fmt (
						"Map : Skipping \"%s\"", line));
					Bell ();
					(void) EmpireMore ();
					break;
				}
			}
	}
}

void FScanRadar (rad)
char *rad;
{
	FeedEmpire (Fmt ("radar %s", rad), PRINT);
	ScanRadar ();
	(void) WaitForPrompt (PRINT);
}

void FeedSatellite ()
{
	FeedEmpire (Fmt ("satellite %s", "*"), PRINT);
	scanning_satellites=True;
	ScanSatellite ();
	scanning_satellites=False;
	(void) WaitForPrompt (PRINT);
}

void ScanRadar ()
{
	int x, y, d, sx, sy;
	char *line, *ptri;
	char des;
	Sector sct;

	if (EmpireStatus () == E_COMMAND)
		return;

	MessageAtEmpire ("scanning radar(s)");

	for (;;)
	{
		if (EmpireStatus () == E_PRINTING)
		{
			line = ReadEmpire (DONT_PRINT);
			if (line == (char *) 0) return;
		}

		if (EmpireStatus () != E_PRINTING)
		{
			PrintAtEmpire ("");
			PrintAtEmpire (line);
			return;
		}

		if (sscanf (line, "%d,%d efficiency %*d%%, max range %d",
			&sx, &sy, &d) != 3) 
		{
		   
		   if ((ptri = index (line, '#')) == (char *) 0)
		      {
			 PrintAtEmpire (Fmt (
			   "ScanRadar: new kind of radar ?\n\"%s\"\n",
					     line));
			 Bell ();
			 (void) EmpireMore ();
			 continue;
		      }

		   if (sscanf (ptri, "#%*d at %d,%d efficiency %*d%%, \
				max range %d", &sx, &sy, &d) != 3)
		      if (sscanf (ptri, "#%*d) at %d,%d efficiency %*d%%, \
				max range %d", &sx, &sy, &d) != 3)
			 {
			    PrintAtEmpire (Fmt (
					   "ScanRadar: nothing? ??\n\"%s\"\n",
						line));
			    Bell ();
			    (void) EmpireMore ();
			    continue;
			 }

		   sct = World (sx, sy, S_DESIG);
		   (void) UpdateSectorDes (sct, '.', False);
		}

		if (d == 0)
		{
			(void) SkipNLines (2, DONT_PRINT);
			continue;
		}

		for (y = sy - d; y <= sy + d; y++)
		{
			ptri = line = ReadEmpire (DONT_PRINT);
			if (line == (char *) 0) return;

			for (x = sx - 2*d; x <= (sx + 2*d); ptri++, x++)
			{
				if (*ptri == '\0')
				{
		  PrintAtEmpire (Fmt (" Trying to scan 0 (x, y) %d\"%s\"\n", x, y, d, line));
					Bell ();
					(void) EmpireMore ();
					break;
				}
				if ((x + y) % 2 == 0)
				{
					des = *ptri;

					if (des == ' ' || des == '0')
						continue;

					sct = World (x, y, S_DESIG);
					if (des >= 'A' && des <= 'Z')
					{
						if (!ShipAtSect (sct, des))
						{
			  PrintAtEmpire (
  	Fmt ( "Warning Enemy ship (%c) at %s detected with no further info",
							    des, CrdStr (sct)));
#ifdef MORE_ON_ENEMY_SHIPS
			  Bell ();
			  (void) EmpireMore ();
#endif
			  
						}

						(void) UpdateSectorDes
							(sct, '.', False);
					}
		  else if (des == '$') {

		    PrintAtEmpire (Fmt
				   ("Satellite detected at %s", CrdStr (sct)));
#ifdef MORE_ON_SATELLITES
		    Bell ();
		    (void) EmpireMore ();
#endif
		  }
					else if (!UpdateSectorDes (sct, des,
								True))
					{
						PrintAtEmpire (
						Fmt ("Skipping \"%s\"", line));
						Bell ();
						(void) EmpireMore ();
					}
				}
			}
		}

		line = ReadEmpire (DONT_PRINT);

		if (line == (char *) 0)
			return;
	}
}

void FScanLookout (where)
     char *where;
{
	FeedEmpire (Fmt ("lookout %s", where), PRINT);
	ScanLookout ();
	(void) WaitForPrompt (PRINT);
}

void FScanCoastWatch (where)
     char *where;
{
	FeedEmpire (Fmt ("coastwatch %s", where), PRINT);
	if (! SkipNLines (2, PRINT))
		return;
	ScanLookout ();
	(void) WaitForPrompt (PRINT);
}

void ScanLookout ()
{

	char *line, *ptri, *tmpptr, savechar, *name;
	int owner, num, x, y;
	char des;
	Sector sct;
	bool err;
	time_t tt, time ();

	MessageAtEmpire ("scanning lookout");

	tt = time ((time_t *) 0);
	for (;;)
	{
		ptri = line = ReadEmpire (DONT_PRINT);
		if (line == (char *) 0) return;

		if (EmpireStatus () != E_PRINTING)
		{
			PrintAtEmpire ("");
			PrintAtEmpire (line);
			return;
		}

		SkipBlank (&ptri);

		if (*ptri == '\n' || *ptri == '\0') continue;

		if (!strncmp (ptri, "Your ", 5))
			/* It's one of my own sectors, just skip it */
			continue;
		
		name = ptri;
		ptri = index (ptri, '(');
		if (ptri == (char *) 0)
		{
			PrintAtEmpire ("Scan L/S/C : no country");
			PrintAtEmpire ( Fmt ("Skipping \"%s\"", line));
			Bell ();
			(void) EmpireMore ();
			continue;
		}
		tmpptr = ptri;

		ptri += 2;
		if (! ScanDigit (&ptri, &owner))
		{
			PrintAtEmpire ("Scan L/S/C : Bad country format");
			PrintAtEmpire ( Fmt ("Skipping \"%s\"", line));
			Bell ();
			(void) EmpireMore ();
			continue;
		}

		do
			tmpptr--;
		while (isspace (*tmpptr));

		savechar = * ++tmpptr;
		* tmpptr = '\0';

		AddCountry (owner, name);

		* tmpptr = savechar;


		if (*ptri != ')')
			ptri = index (ptri, ')');

		if (ptri == (char *) 0)
		{
			PrintAtEmpire ("Scan L/S/C : Bad country format");
			PrintAtEmpire ( Fmt ("Skipping \"%s\"", line));
			Bell ();
			(void) EmpireMore ();
			continue;
		}
		ptri++; /* skip ')' */

		SkipBlank (&ptri);

		tmpptr = index (ptri, '#');

		if (tmpptr == (char *) 0)
		{
			/* it's land */
			int eff, civ, mil;

			des = CharOfDesigName (ptri);
			if (des == '\0')
			{
				PrintAtEmpire (
				"Scan L/S/C : Unknown Designation ?");
				PrintAtEmpire ( Fmt ("Skipping \"%s\"", line));
				Bell ();
				(void) EmpireMore ();
				continue;
			}
			ptri += strlen  (DesignName (des));
			err = ! ScanDigit (&ptri, &eff) || *ptri != '%';
			ptri++; /* Skip '%' */

			SkipBlank (&ptri);

			if (err || (strncmp (ptri, "efficient", 9) != 0))
			{
				PrintAtEmpire ("Scan L/S/C : Bad eff Format");
				PrintAtEmpire ( Fmt ("Skipping \"%s\"", line));
				Bell ();
				(void) EmpireMore ();
				continue;
			}
			ptri += 9;

			SkipBlank (&ptri);

			civ = 0;
			mil = 0;
			for (;;)
			{
				if (strncmp (ptri, "with approx", 11) == 0)
				{
					ptri += 12;
					err = ! ScanDigit (&ptri, &num);
					ptri++;
					if (!err && *ptri == 'c')
						civ = num == 0 ? 1 : num;
					else if (!err && *ptri == 'm')
						mil = num == 0 ? 1 : num;
					else
					{
						PrintAtEmpire (
					"Scan L/S/C : Bad Lookout format");
						PrintAtEmpire (
					Fmt ("Skipping \"%s\"", line));
						Bell ();
						(void) EmpireMore ();
						break;
					}
					ptri += 4;
				}
				else
					break;
			}

			SkipBlank (&ptri);
			if  (*ptri != '@' ||
					sscanf (ptri, "@ %d,%d", &x, &y) != 2)
			{
				PrintAtEmpire (
					"Scan L/S/C : Unknown @ format ?");
				PrintAtEmpire (Fmt ( "Skipping \"%s\"", line));
				Bell ();
				(void) EmpireMore ();
				continue;
			}

			sct = World (x, y, S_RESOU);

			if (s_time (sct) == tt)
				/* Already got it */
				continue;

			if (s_owned (sct))
			{
				/* Must have been captured */

				PrintAtEmpireMode (
				     Fmt ("Your sector at %s has been captured",
				     CrdStr (sct)), BOLD);
/*				Bell (); TARL - Don't ring bell. Annoying. */
				DEL_INFO (sct);
				SET_INFO_RES (sct); /* save resources */
				set_owner (sct, UNKNOWN_CNUM);
			}
			/* This is most uptodate info overwrite anythin else */
			set_des (sct, des);
			set_owner (sct, owner);
			set_eff (sct, eff);
			set_civ (sct, civ);
			set_mil (sct, mil);
			set_time (sct, tt);
			SET_INFO_LOO (sct);
			PrintAtEmpire (line);
		}
		else
		{
			/* it's a ship */

			if (sscanf (tmpptr, "#%d @ %d,%d", &num, &x, &y) != 3)
			{
				PrintAtEmpire (
					"Scan L/S/C : Unknown ship format ?");
				PrintAtEmpire (Fmt ("Skipping \"%s\"", line));
				Bell ();
				(void) EmpireMore ();
				continue;
			}

			do
				tmpptr--;

			while (isspace (*tmpptr));

			savechar = * ++tmpptr;
			* tmpptr = '\0'; /* leave's in ptri shipname */

			if (AddEnemyShip (x, y, num, owner, ptri))
			{
				* tmpptr = savechar;
				PrintAtEmpire (line);
			}
			/* else ship already on list and at same position
			 * don't tell player what he already knows
			 */
		}
	}
}

void FScanReport (who)
     char *who;
{
	FeedEmpire (Fmt ("report %s", who), PRINT);
	ScanReport ();
	(void) WaitForPrompt (PRINT);
}

void ScanReport ()
{
	char *line, *ptri, *tmpptr;
	int nr;
	extern WinInfo empire_win;

	MessageAtEmpire ("searching country names");

	if (! SkipNLines (1, DONT_PRINT))
		return;

	for (;;)
	{
		ptri = line = ReadEmpire (DONT_PRINT);

		if (line == (char *) 0)
			return;

		if (EmpireStatus () != E_PRINTING)
		{
			PrintAtEmpire ("");
			PrintAtEmpire (line);
			return;
		}

		SkipBlank (&ptri);

		if (! ScanDigit (& ptri, & nr))
		{
			PrintAtEmpire ("Bad relation format");
			PrintAtEmpire (Fmt ("Skipping \"%s\"", line));
			Bell ();
			(void) EmpireMore ();
			continue;
		}

		SkipBlank (& ptri);

		tmpptr = line + 21;

		do
			tmpptr--;
		while (isspace (*tmpptr));

		* ++ tmpptr = '\0';

		AddCountry (nr, ptri);
	}
}

void FScanRelations (cnum)
     int cnum;
{
	FeedEmpire (Fmt ("relations %d", cnum), PRINT);
	ScanRelations ();
	(void) WaitForPrompt (PRINT);
}

void ScanRelations ()
{
	char *line, *ptri, *tmpptr;
	int nr;
	bool err;
	extern WinInfo empire_win;

	MessageAtEmpire ("searching country names");

	(void) SkipNLines (3, DONT_PRINT);

	for (;;)
	{
		ptri = line = ReadEmpire (DONT_PRINT);
		if (line == (char *) 0) return;

		if (EmpireStatus () != E_PRINTING)
		{
			PrintAtEmpire ("");
			PrintAtEmpire (line);
			return;
		}

		SkipBlank (&ptri);

		err = ! ScanDigit (& ptri, &nr);


		ptri = index (ptri, ')') + 1;

		if (err || ptri == (char *) 0)
		{
			PrintAtEmpire ("Bad relation format");
			PrintAtEmpire (Fmt ("Skipping \"%s\"", line));
			Bell ();
			(void) EmpireMore ();
			continue;
		}

		SkipBlank (& ptri);

		tmpptr = line + 26;

		do
			tmpptr--;
		while (isspace (*tmpptr));

		* ++ tmpptr = '\0';

		AddCountry (nr, ptri);
	}
}

void FScanSpy (where)
     char *where;
{
	FeedEmpire (Fmt ("spy %s", where), PRINT);
	ScanSpy ();
	(void) WaitForPrompt (PRINT);
}
	
bool ScanSpyLine (line)
     char * line;
{
	bool err;
	int x, y;
	Sector sct;
	int own, eff;
	int nr;

	err = sscanf (line , " %d,%d ", &x, &y) != 2;

	if (! err)
	{
		sct = World (x, y, S_RESOU);
			/*
			 *	Spyplanes also reports your own sectors
			 */

		if (s_owned (sct))
			return True;

		nr = sscanf (line,
	       " %*d,%*d %c %d %d %hd %hd %hd %hd %hd %hd %hd",
	       & sct-> des,
	       & own,
	       & eff,
	       & sct-> sct2-> civ,
	       & sct-> sct2-> mil,
	       & sct-> sct2-> qua [D_SHE],
	       & sct-> sct2-> qua [D_GUN],
	       & sct-> sct2-> qua [D_IRO],
	       & sct-> sct2-> qua [D_PET],
	       & sct-> sct2-> qua [D_FOO]);
	}

	if (err || nr != 10)
		return False;

	set_owner (sct, own);
	set_eff (sct, eff);
	SET_INFO_SPY (sct);
	SetSectorTime (sct);
	DrawSector (sct);

	return True;
}

void ScanSpy ()
{
	char *line;

	MessageAtEmpire ("scanning spy output");

	do {
		line = ReadEmpire (PRINT);

		if (EmpireStatus () == E_COMMAND)
			return;
	}
	while (! StrEQ (line, "   sect   type own") &&
	       ! StrEQ (line,  "  sect   type own"));

	for (;;)
	{
		line = ReadEmpire (PRINT);
		if (line == (char *) 0)
			return;
		
		if (EmpireStatus () != E_PRINTING)
		{
			PrintAtEmpire ("");
			PrintAtEmpire (line);
			return;
		}

		if (! ScanSpyLine (line))
		{
			Message ("Spy failed");
			Bell ();
			(void) EmpireMore ();
			ClearMes ();
			continue;
		}
	}
}

bool SetOffset (country)
     char *country;
{
	char *line;

	FeedEmpire (Fmt ("offset %s", country), PRINT);
	for (;;) {
		line = ReadEmpire (PRINT);
		if (line == (char *) 0)
			return False;
		if (EmpireStatus () == E_COMMAND) {
			PrintAtEmpire (line);
			return False;
		}
		
		if (! strncmp (line, "No such country", 15)) {
			fprintf (stderr, "No such country : %s\n", country);
			leave ();
		}

		if (! strncmp (line, "Old offset was", 14)) {
			(void) WaitForPrompt (PRINT);
			deity = True;
			return True;
		}
	}
}

/*
 * new code mta Fri Nov 30 22:26:02 EET 1990
 */
void ScanSatellite ()
{
   register int x, y, start_x, end_x, start_y, end_y;
   int d, sx, sy, off_sector;
   char *line, *ptri;
   char des;
   Sector sct;
   
   if (EmpireStatus () == E_COMMAND)
      return;
   
   MessageAtEmpire ("scanning satellite(s)");
   
   for (;;)
      {
	 if (EmpireStatus () == E_PRINTING)
	    {
	       line = ReadEmpire (DONT_PRINT);
	       if (line == (char *) 0) return;
	    }
	 
	 if (EmpireStatus () != E_PRINTING)
	    {
	       PrintAtEmpire ("");
	       PrintAtEmpire (line);
	       return;
	    }
	 
	 if (!strncmp (line, "Satellite", 9))
	    continue;
	 
	 if (!strncmp (line, "(c) 1989 Imaginative Images Inc.", 32))
	    continue;
	 
	 if (sscanf (line, 
	     "landsat %*[12] #%*d at %d,%d efficiency %*d%%, max range %d",
		     &sx, &sy, &d) != 3) {
	    ptri = index(line, '#');
	    if (sscanf (ptri, 
		    /*	        "spysat %*[12] #%*d at %d,%d efficiency %*d%%, max range %d",*/
		"#%*d at %d,%d efficiency %*d%%, max range %d",
			&sx, &sy, &d) != 3) {
	       PrintAtEmpire (Fmt (
		"ScanSatellite: new kind of satellite ?\n\"%s\"\n",
				   line));
	       Bell ();
	       (void) EmpireMore ();
	       continue;
	    } else {
	       MessageAtEmpire("scanning spysat");
	       /* skip:
		*      Satellite sector report
		*      date
		*      sect   type own eff   civ  mil  shl  gun iron  pet  food
		*/
	       SkipNLines(1, PRINT); 
	       SkipNLines(3, DONT_PRINT); 
	       line = ReadEmpire (DONT_PRINT);
	       while( ScanSpyLine(line) )
		  line = ReadEmpire (DONT_PRINT);
	       /* skip:
		*      empty
		*      Satellite ship report
		*      date
		*      own  shp# ship type         sector   eff
		*/
	       SkipNLines(1, DONT_PRINT);
	       SkipNLines(1, PRINT);
	       SkipNLines(1, DONT_PRINT);
	       line = ReadEmpire (DONT_PRINT);
	  off_sector = (char *)StrStr(line, "sector") - line;
	       line = ReadEmpire (DONT_PRINT);
	       while ( AddSatelliteShip(line, off_sector) )
		  line = ReadEmpire (DONT_PRINT);
	       /* skip:
		*      empty
		*      Satellite radar report
		*/
	       SkipNLines(1, DONT_PRINT);
	       SkipNLines(1, PRINT);
	       
	    } 
	 } else {
	    MessageAtEmpire("scanning landsat");
	    SkipNLines(1, DONT_PRINT);
	 }
	 
	 sct = World (sx, sy, S_DESIG);
	 (void) UpdateSectorDes (sct, '.', False);
	 
	 
	 if (d == 0)
	    {
	       (void) SkipNLines (2, DONT_PRINT);
	       continue;
	    }

	 /* Don't try to scan more than width of world */
	 if ( d < MAX_X/4 ) {
	    start_x = sx - (2 * d);
	    end_x = sx + (2 * d);
	 } else {
	    start_x = sx - (MAX_X/2);
	    end_x = sx + (MAX_X/2) - 1;
	 }
	 
 
 	 if (d < MAX_Y/2) {	/* TARL - Don't scan more than height */
 	   start_y = sy - d;	/* of world. */
 	   end_y = sy + d;
 	 } else {
 	   start_y = sy - (MAX_Y/2);
 	   end_y = sy + (MAX_Y/2) - 1;
 	 }
  	 
 	 for (y = start_y; y <= end_y; y++)
	    {
	       ptri = line = ReadEmpire (DONT_PRINT);
	       if (line == (char *) 0) return;
	       
	       for (x = start_x; x <= end_x; ptri++, x++)
		  {
		     if (*ptri == '\0')
			{
			   PrintAtEmpire (Fmt (" Trying to scan 0 (%d, %d) %d\"%s\"\n", x, y, d, line));
			   Bell ();
			   (void) EmpireMore ();
			   break;
			}
		     if ((x + y) % 2 == 0)
			{
			   des = *ptri;
			   
			   if (des == ' ' || des == '0')
			      continue;
			   
			   sct = World (x, y, S_DESIG);
			   if (des >= 'A' && des <= 'Z')
			      {
				 if (!ShipAtSect (sct, des))
				    {
				       PrintAtEmpire (
						      Fmt ( "Warning Enemy ship (%c) at %s detected with no further info",
							   des, CrdStr (sct)));
/* TARL - be quiet. */
#ifdef MORE_ON_ENEMY_SHIPS
			  Bell ();
			  (void) EmpireMore ();
#endif
				    }
				 
				 (void) UpdateSectorDes
				    (sct, '.', False);
			      }
		  else if (des == '$') {
			      PrintAtEmpire (Fmt ("Satellite detected at %s", CrdStr (sct)));
/* TARL - Be quiet */
#ifdef MORE_ON_ENEMY_SATELLITES
		    Bell ();
		    (void) EmpireMore ();
#endif
		  }
			   else if (!UpdateSectorDes (sct, des,
						      (des == '?')? False:True))
			      {
				 PrintAtEmpire (
						Fmt ("Skipping \"%s\"", line));
				 Bell ();
				 (void) EmpireMore ();
			      }
			}
		  }
	    }
	 
	 line = ReadEmpire (DONT_PRINT);
	 
	 if (line == (char *) 0)
	    return;
      }
}


AddSatelliteShip (line, sector)
     char *line;
     int sector;
{
   bool err;
   int x, y;
   Sector sct;
   int own, eff, shp;
   char type[128], *tmpptr;
   char *coords = line+sector;
   
   if (strlen(line) < 20)
      return False;

   SkipBlank( &line );
   err = !ScanDigit( &line, &own);
   err |= !ScanDigit( &line, &shp);

   SkipBlank( &line );

   if (! err && own != my_cnum)
      {
	 tmpptr = line+17;	/* was 18, off by one */
	 do
	    tmpptr--;
	 while (isspace (*tmpptr));
	 *(tmpptr+1) = '\0';
	 sscanf(coords, "%d,%d %d%%", &x, &y, &eff);
	 if (AddEnemyShip(x, y, shp, own, line))
	    PrintAtEmpire(Fmt ("Enemy (%d) ship (%s) at %d,%d", own, line, x, y));
      }

   if ( err )
      return False;
   else
      return True;
}

